#Detache capsomer model from its capsid

#This tool displaces a capsomer away from its capsid's center of mass inputing only the base capsomer model.
#Usage: ./detachmdl.sh {file1} {distance}
#file1 is the base model capsomer, either in pdb or pqr format. The file MUST include the BIOMT transformation matrix, which is usually included at the header under "REMARK 350".
#displacement is the desired displacement of the base capsomer away from the capsid's center of mass.
#The output will have the form of: 
#
#         "  detach{distance}_{file1}  "
#
#The script aditionally creates 1 temporary files with the names of:
#
#         "  {file1}.temp1
#

if [ $# != 2 ]
then
        echo "can't run, it need parameter... "
        echo 'Usage: ./detach.sh mdle1 dist '
        exit
fi

d=$2
#####get mass center of original capsomer

line=$(awk -v num=0 -v sumx=0 -v sumy=0 -v sumz=0 -v num=0 -v mt=0 ' { ATOM=substr($0,14,1) } { if(ATOM=="C") { mn=12.0107 } else if(ATOM=="N") { mn=14.0067 } else if(ATOM=="O") { mn=15.9994 } else if(ATOM=="S") { mn=32.065 } else { mn=1.008 } if($1=="ATOM" || $1=="HETATM") {x=substr($0,31,8);y=substr($0,39,8);z=substr($0,47,8);mt=mt+mn;sumx=sumx+(x*mn);sumy=sumy+(y*mn);sumz=sumz+(z*mn);num=num+1 } } END {print sumx/mt,sumy/mt,sumz/mt} ' $1 )  

for ((n=1;n<=9999;n++))
do 
#####split the transformation matrix in x, y, and z coordinates:
###new x

a=$(grep "BIOMT1" $1 | awk -v n=$n '{ if($4==n ) print $5 }')
b=$(grep "BIOMT1" $1 | awk -v n=$n '{ if($4==n ) print $6 }')
c=$(grep "BIOMT1" $1 | awk -v n=$n '{ if($4==n ) print $7 }')
dx=$(grep "BIOMT1" $1 | awk -v n=$n '{ if($4==n ) print $8 }')

###new y

d=$(grep "BIOMT2" $1 | awk -v n=$n '{ if($4==n ) print $5 }')
e=$(grep "BIOMT2" $1 | awk -v n=$n '{ if($4==n ) print $6 }')
f=$(grep "BIOMT2" $1 | awk -v n=$n '{ if($4==n ) print $7 }')
dy=$(grep "BIOMT2" $1 | awk -v n=$n '{ if($4==n ) print $8 }')

###new z

g=$(grep "BIOMT3" $1 | awk -v n=$n '{ if($4==n ) print $5 }')
h=$(grep "BIOMT3" $1 | awk -v n=$n '{ if($4==n ) print $6 }')
i=$(grep "BIOMT3" $1 | awk -v n=$n '{ if($4==n ) print $7 }')
dz=$(grep "BIOMT3" $1 | awk -v n=$n '{ if($4==n ) print $8 }')

#####condition to exit when done

N=$(grep "BIOMT1" $1 | awk -v n=$n '{if ($4==n) print n }')

if [ -z $N ]
then
exit
fi

#####apply the transformation matrix to base capsomer's center of mass. 

echo $line | awk -v a=$a -v b=$b -v c=$c -v dx=$dx -v d=$d -v e=$e -v f=$f -v dy=$dy -v g=$g -v h=$h -v i=$i -v dz=$dz '{print ($1*a)+($2*b)+($3*c)+dx, ($1*d)+($2*e)+($3*f)+dy, ($1*g)+($2*h)+($3*i)+dz}'

done | awk '{n=n+1}{ print "MODEL", n, $0 }' > ${1}.temp1

#####calculating the average of all centers of mass to get the capsid's center of mass. 

x1=$(awk '{mx=$3;n=n+1;sumx=sumx+mx} END {print sumx/n}' ${1}.temp1)
y1=$(awk '{my=$4;n=n+1;sumy=sumy+my} END {print sumy/n}' ${1}.temp1)
z1=$(awk '{mz=$5;n=n+1;sumz=sumz+mz} END {print sumz/n}' ${1}.temp1)

echo "Center of mass of virus:" $x1 $y1 $z1
echo "Center of mass of base model capsomer:" $line

#####find change in position between model's center of mass and capsid's center of mass.

dx=$(echo $line $x1 | awk '{ print ($1-$4) }' )
dy=$(echo $line $y1 | awk '{ print ($2-$4) }' )
dz=$(echo $line $z1 | awk '{ print ($3-$4) }' )

#####calculate normalization factor for vectors found previously

norm=$(echo $dx $dy $dz | awk '{ print ((($1)^2)+(($2)^2)+(($3)^2))^0.5 }' )

echo "Normalization factor:" $norm

#####find unit vector

uvx=$(echo $norm $dx | awk '{print $2/$1}')
uvy=$(echo $norm $dy | awk '{print $2/$1}')
uvz=$(echo $norm $dz | awk '{print $2/$1}')

echo "Direction unit vector components:" $uvx $uvy $uvz

##### find the displacement in x, y, and z axis. 

ddx=$(echo $uvx $2 | awk '{print $1*$2}')
ddy=$(echo $uvy $2 | awk '{print $1*$2}')
ddz=$(echo $uvz $2 | awk '{print $1*$2}')

echo "Displacement vector components:" $ddx $ddy $ddz

##### print final coordinates.

awk -v ddx=$ddx -v ddy=$ddy -v ddz=$ddz '{if($1 == "ATOM" || $1 == "HETATM")  printf("%s%8.3f%8.3f%8.3f%s\n",substr($0,1,30), substr($0,31,8)+ddx,substr($0,39,8)+ddy,substr($0,47,8)+ddz,substr($0,55,100)); else print $0 }' $1 > detach${2}_${1}

rm ${1}.temp1


