
#This tool displaces all capsomers inside a virus capsid away from the capsid's center of mass by an equal distance by only inputting the base capsomer model. 
#Usage: ./expand.sh {file1} {distance}
#file1 is the capsomer base model, either in pdb or pqr format. The file MUST include the BIOMT transformation matrix, which is usually included at the header under "REMARK 350".
#distance is the desired displacement of the capsomers away from the capsid's center of mass.
#The script will have as many outputs as the capsid has capsomers and they will have the form of: 
#
#         "  {num}_exp{distance}_{file1}  ", where {num} is the assigned capsomer number to that file. It usually ranges from 1 to 60.   
#
#The script aditionally creates 4 temporary files with the names of:
#
#         "  {file1}.temp{1,2,3,4}
#


if [ $# != 2 ]
then
        echo "can't run, it need parameter... "
        echo 'Usage: ./expand.sh mdle1 dist '
        exit
fi


d=$2
#get mass center of original capsomer

line=$(awk -v num=0 -v sumx=0 -v sumy=0 -v sumz=0 -v num=0 -v mt=0 ' { ATOM=substr($0,14,1) } { if(ATOM=="C") { mn=12.0107 } else if(ATOM=="N") { mn=14.0067 } else if(ATOM=="O") { mn=15.9994 } else if(ATOM=="S") { mn=32.065 } else { mn=1.008 } if($1=="ATOM" || $1=="HETATM") {x=substr($0,31,8);y=substr($0,39,8);z=substr($0,47,8);mt=mt+mn;sumx=sumx+(x*mn);sumy=sumy+(y*mn);sumz=sumz+(z*mn);num=num+1 } } END {print sumx/mt,sumy/mt,sumz/mt} ' $1 )  

for ((n=1;n<=9999;n++))
do 

########## LOOP 1: Find mass center of whole capsid. 
#####split the transformation matrix in x, y, and z coordinates:
###new x

a=$(grep "BIOMT1" $1 | awk -v n=$n '{ if($4==n ) print $5 }')
b=$(grep "BIOMT1" $1 | awk -v n=$n '{ if($4==n ) print $6 }')
c=$(grep "BIOMT1" $1 | awk -v n=$n '{ if($4==n ) print $7 }')
dx=$(grep "BIOMT1" $1 | awk -v n=$n '{ if($4==n ) print $8 }')

###new y

d=$(grep "BIOMT2" $1 | awk -v n=$n '{ if($4==n ) print $5 }')
e=$(grep "BIOMT2" $1 | awk -v n=$n '{ if($4==n ) print $6 }')
f=$(grep "BIOMT2" $1 | awk -v n=$n '{ if($4==n ) print $7 }')
dy=$(grep "BIOMT2" $1 | awk -v n=$n '{ if($4==n ) print $8 }')

###new z

g=$(grep "BIOMT3" $1 | awk -v n=$n '{ if($4==n ) print $5 }')
h=$(grep "BIOMT3" $1 | awk -v n=$n '{ if($4==n ) print $6 }')
i=$(grep "BIOMT3" $1 | awk -v n=$n '{ if($4==n ) print $7 }')
dz=$(grep "BIOMT3" $1 | awk -v n=$n '{ if($4==n ) print $8 }')

#####condition to exit loop 1 when done

N=$(grep "BIOMT1" $1 | awk -v n=$n '{if ($4==n) print n }')

if [ -z $N ]
then
exit
fi

#####apply the transformation matrix to base capsomer's center of mass. 

echo $line | awk -v a=$a -v b=$b -v c=$c -v dx=$dx -v d=$d -v e=$e -v f=$f -v dy=$dy -v g=$g -v h=$h -v i=$i -v dz=$dz '{print ($1*a)+($2*b)+($3*c)+dx, ($1*d)+($2*e)+($3*f)+dy, ($1*g)+($2*h)+($3*i)+dz}'

done | awk '{n=n+1}{ print "MODEL", n, $0 }' > ${1}.temp1 

#####calculating the average of all centers of mass to get the capsid's center of mass. 

x1=$(awk '{mx=$3;n=n+1;sumx=sumx+mx} END {print sumx/n}' ${1}.temp1)
y1=$(awk '{my=$4;n=n+1;sumy=sumy+my} END {print sumy/n}' ${1}.temp1)
z1=$(awk '{mz=$5;n=n+1;sumz=sumz+mz} END {print sumz/n}' ${1}.temp1)

echo "Center of Mass of Virus:" $x1 $y1 $z1
echo "Center of Mass of Capsomer base model:" $line

#####find change in position between all model's center of mass and capsid's center of mass.

awk -v x1=$x1 -v y1=$y1 -v z1=$z1 '{n=n+1} { print "MODEL", n, $3-x1, $4-y1, $5-z1 }' ${1}.temp1 > ${1}.temp2

#####calculate normalization factor for vectors found previously

norm=$(awk '{ if($2==1) {print (($3^2)+($4^2)+($5^2))^.5 } }' ${1}.temp2)

echo "Virus Radius:" $norm

#####find x, y, and z components of unit vectors 

awk -v n=$n -v norm=$norm '{n=n+1} {print "MODEL", n, $3/(norm), $4/(norm), $5/(norm) }' ${1}.temp2  > ${1}.temp3

#####find displacement vector for each model

awk -v n=$n -v d=$d '{ n=n+1 } {print "MODEL", n, $3*d, $4*d, $5*d }' ${1}.temp3 > ${1}.temp4

#####generate new models by applying transformation matrix to base model and then the displacement found previously.

for ((n=1;n<=9999;n++))
do

#new x

a=$(grep "BIOMT1" $1 | awk -v n=$n '{ if($4==n ) print $5 }')
b=$(grep "BIOMT1" $1 | awk -v n=$n '{ if($4==n ) print $6 }')
c=$(grep "BIOMT1" $1 | awk -v n=$n '{ if($4==n ) print $7 }')
dx=$(grep "BIOMT1" $1 | awk -v n=$n '{ if($4==n ) print $8 }')

#new y

d=$(grep "BIOMT2" $1 | awk -v n=$n '{ if($4==n ) print $5 }')
e=$(grep "BIOMT2" $1 | awk -v n=$n '{ if($4==n ) print $6 }')
f=$(grep "BIOMT2" $1 | awk -v n=$n '{ if($4==n ) print $7 }')
dy=$(grep "BIOMT2" $1 | awk -v n=$n '{ if($4==n ) print $8 }')

#new z

g=$(grep "BIOMT3" $1 | awk -v n=$n '{ if($4==n ) print $5 }')
h=$(grep "BIOMT3" $1 | awk -v n=$n '{ if($4==n ) print $6 }')
i=$(grep "BIOMT3" $1 | awk -v n=$n '{ if($4==n ) print $7 }')
dz=$(grep "BIOMT3" $1 | awk -v n=$n '{ if($4==n ) print $8 }')

#displacement vector

ddx=$(awk -v n=$n '{ if($2==n) {print $3 }}' ${1}.temp4)
ddy=$(awk -v n=$n '{ if($2==n) {print $4 }}' ${1}.temp4)
ddz=$(awk -v n=$n '{ if($2==n) {print $5 }}' ${1}.temp4)

#condition to exit when done

S=$(grep "BIOMT1" $1 | awk -v n=$n '{if ($4==n) print n }')

if [ -z $S ]
then
rm ${1}.temp1
rm ${1}.temp2
rm ${1}.temp3
rm ${1}.temp4
exit
fi

##### print final coordinates

awk -v a=$a -v b=$b -v c=$c -v dx=$dx -v d=$d -v e=$e -v f=$f -v dy=$dy -v g=$g -v h=$h -v i=$i -v dz=$dz -v ddx=$ddx -v ddy=$ddy -v ddz=$ddz '{if($1 == "ATOM" || $1 == "HETATM") { x=substr($0,31,8);y=substr($0,39,8);z=substr($0,47,8);printf("%s%8.3f%8.3f%8.3f%s\n",substr($0,1,30),((x*a)+(y*b)+(z*c))+dx+ddx,((x*d)+(y*e)+(z*f))+dy+ddy,((x*g)+(y*h)+(z*i))+dz+ddz,substr($0,55,100))} else print $0 }' $1 > ${n}_exp${2}_$1

echo "MODEL" $n

done

