#Generate capsid structure from a base capsomer.

#This tool generates a whole viral capsid structure by inputting only the case capsomer model.
#Usage: ./generate.sh file 1
#file 1 is the base model capsomer, either in pdb or pqr format. The file MUST include the BIOMT transformation matrix, which is usually included at the header under "REMARK 350".
#The script will have as many outputs as the capsid has capsomers and they will have the form of: 
#
#         "  {num}_{file1}  ", where {num} is the assigned capsomer number to that file. It usually ranges from 1 to 60. 
#


if [ $# != 1 ]
then
        echo "can't run, it need parameter... "
        echo 'Usage: ./generate.sh mdle1 '
        exit
fi

for ((n=1;n<=9999;n=n+1))

do 
#####split the transformation matrix in x, y, and z coordinates:
#new x

a=$(awk -v n=$n '{ if($3 ~ /BIOMT1/ && $4==n ) print $5 }' $1)
b=$(awk -v n=$n '{ if($3 ~ /BIOMT1/ && $4==n ) print $6 }' $1)
c=$(awk -v n=$n '{ if($3 ~ /BIOMT1/ && $4==n ) print $7 }' $1)
dx=$(awk -v n=$n '{ if($3 ~ /BIOMT1/ && $4==n ) print $8 }' $1)

#new y

d=$(awk -v n=$n '{ if($3 ~ /BIOMT2/ && $4==n ) print $5 }' $1)
e=$(awk -v n=$n '{ if($3 ~ /BIOMT2/ && $4==n ) print $6 }' $1)
f=$(awk -v n=$n '{ if($3 ~ /BIOMT2/ && $4==n ) print $7 }' $1)
dy=$(awk -v n=$n '{ if($3 ~ /BIOMT2/ && $4==n ) print $8 }' $1)

#new z

g=$(awk -v n=$n '{ if($3 ~ /BIOMT3/ && $4==n ) print $5 }' $1)
h=$(awk -v n=$n '{ if($3 ~ /BIOMT3/ && $4==n ) print $6 }' $1)
i=$(awk -v n=$n '{ if($3 ~ /BIOMT3/ && $4==n ) print $7 }' $1)
dz=$(awk -v n=$n '{ if($3 ~ /BIOMT3/ && $4==n ) print $8 }' $1)

#####condition to exit when done

N=$(grep "BIOMT1" $1 | awk -v n=$n '{if ($4==n) print n }')

if [ -z $N ]
then
exit
fi

#####apply transformation matrix to the base model capsomer

awk -v a=$a -v b=$b -v c=$c -v dx=$dx -v d=$d -v e=$e -v f=$f -v dy=$dy -v g=$g -v h=$h -v i=$i -v dz=$dz '{if($1 == "ATOM" || $1 == "HETATM") { x=substr($0,31,8);y=substr($0,39,8);z=substr($0,47,8);printf("%s%8.3f%8.3f%8.3f%s\n",substr($0,1,30),((x*a)+(y*b)+(z*c))+dx,((x*d)+(y*e)+(z*f))+dy,((x*g)+(y*h)+(z*i))+dz,substr($0,55,100))} else print $0 }' $1 > ${n}_$1

echo "MODEL" $n

done

