#Calculate center of mass of a virus capsid

#This tool is used to calculate the center of mass of a capsid by only inputting the capsomer base model. 
#Usage: ./masscentcapsid.sh {file1}
#file1 is the capsomer base model, either in pqr of pdb format. The file MUST include the BIOMT transformation matrix, which is usually included at the header under "REMARK 350".
#the output will have the form of:
#
#             "  (x_coordinate) (y_coordinate) (z_coordinate)  " 
#

if [ $# != 1 ]
then
        echo "can't run, it need parameter... "
        echo 'Usage: ./masscentcapsid.sh file1 '
        exit
fi

#get mass center of original capsomer. Current atoms: C, N, O, S.

line=$(awk -v num=0 -v sumx=0 -v sumy=0 -v sumz=0 -v num=0 -v mt=0 ' { ATOM=substr($0,14,1) } { if(ATOM=="C") { mn=12.0107 } else if(ATOM=="N") { mn=14.0067 } else if(ATOM=="O") { mn=15.9994 } else if(ATOM=="S") { mn=32.065 } else { mn=1.008 } if($1=="ATOM" || $1=="HETATM") {x=substr($0,31,8);y=substr($0,39,8);z=substr($0,47,8);mt=mt+mn;sumx=sumx+(x*mn);sumy=sumy+(y*mn);sumz=sumz+(z*mn);num=num+1 } } END {print sumx/mt,sumy/mt,sumz/mt} ' $1 )  

for ((n=1;n<=9999;n++))

do 

##### split transformation matrix in x, y, and z coordinates:
#new x

a=$(grep "BIOMT1" $1 | awk -v n=$n '{ if($4==n ) print $5 }')
b=$(grep "BIOMT1" $1 | awk -v n=$n '{ if($4==n ) print $6 }')
c=$(grep "BIOMT1" $1 | awk -v n=$n '{ if($4==n ) print $7 }')
dx=$(grep "BIOMT1" $1 | awk -v n=$n '{ if($4==n ) print $8 }')

#new y

d=$(grep "BIOMT2" $1 | awk -v n=$n '{ if($4==n ) print $5 }')
e=$(grep "BIOMT2" $1 | awk -v n=$n '{ if($4==n ) print $6 }')
f=$(grep "BIOMT2" $1 | awk -v n=$n '{ if($4==n ) print $7 }')
dy=$(grep "BIOMT2" $1 | awk -v n=$n '{ if($4==n ) print $8 }')

#new z

g=$(grep "BIOMT3" $1 | awk -v n=$n '{ if($4==n ) print $5 }')
h=$(grep "BIOMT3" $1 | awk -v n=$n '{ if($4==n ) print $6 }')
i=$(grep "BIOMT3" $1 | awk -v n=$n '{ if($4==n ) print $7 }')
dz=$(grep "BIOMT3" $1 | awk -v n=$n '{ if($4==n ) print $8 }')

#####condition to exit when done

s=$(echo $a $b $c $d $e $f $g $h $i | awk ' { s=($1+$2+$3+$4+$5+$6+$7+$8+$9);if(s == 0) { print 0 }else print 1 } ')

if [ $s -eq 0 ]
then
exit
fi

#####apply rotation matrix to the base model capsomer's center of mass.

echo $line | awk -v a=$a -v b=$b -v c=$c -v dx=$dx -v d=$d -v e=$e -v f=$f -v dy=$dy -v g=$g -v h=$h -v i=$i -v dz=$dz  '{ print ($1*a)+($2*b)+($3*c)+dx, ($1*d)+($2*e)+($3*f)+dy, ($1*g)+($2*g)+($3*i)+dz }'

#####calculating the average of all centers of mass to get the capsid's center of mass. 

done | awk '{ mx=$1;my=$2;mz=$3;n=n+1;sumx=sumx+mx;sumy=sumy+my;sumz=sumz+mz } END { print sumx/n, sumy/n, sumz/n }' 
