#Displace a protein structure across a perpendicular line. 

#This tool displaces a protein structure with reference to another one across a line that's perpendicular to the distance between them. 
#Usage: ./perpshiftpdb_pqr.sh {file1} {file2} {dispalcement}
#file1 is the reference strucutre. file2 is the structure to be displaced. The structures must be in pdb or pqr format.
#displacement is the desired displacement distance.
#The output will have the form of: 
#
#             "  perp_{displacement}_{file2}  " 
#
#The scripts " masscentpdb_pqr.sh " and " shift.sh " MUST be in the same folder as this tool.
#

if [ $# != 3 ]
then
        echo "can't run, parameters needed. "
        echo "Usage: ./perpshiftpdb_pqr.sh file1 file2 dist"
        exit
fi

file1=$1
file2=$2
dist=$(echo $3 |awk '{print ($1^2)^.5}')               #### Find the absolute value of the distance. 

##### Depending if the input distance is negative or positive, the shift will have opposite direction
if [ $3 -ge 0 ]
then
ang=$(echo ' 3.1415926 2 ' |awk '{print $1/$2}')
else
ang=$(echo ' 3.1415926 2 ' |awk '{print -$1/$2}')
fi

#####find centers of mass of both capsomers
x1=$(./masscentpdb_pqr.sh $1 |awk '{print $1}')
y1=$(./masscentpdb_pqr.sh $1 |awk '{print $2}')

x2=$(./masscentpdb_pqr.sh $2 |awk '{print $1}')
y2=$(./masscentpdb_pqr.sh $2 |awk '{print $2}')

#####find change in position vector components between the two centers of mass
xv=$(echo $x2 $x1 |awk '{print $1-$2}')
yv=$(echo $y2 $y1 |awk '{print $1-$2}')

#####find vector magnitude/normalization factor
vec=$(echo $xv $yv |awk '{print (($1^2)+($2^2))^.5 }' )

#####find unit vector components 
xf=$(echo $xv $vec |awk '{print $1/$2}' )
yf=$(echo $yv $vec |awk '{print $1/$2}' )

#####find components of displacement vector away from capsomer
xs=$(echo $xf $dist |awk '{print $1*$2}' )
ys=$(echo $yf $dist |awk '{print $1*$2}' ) 

#####rotate displacement vector 90/-90 degrees 
drx=$(echo $xs $ys $ang |awk '{ xn=(($1*cos($3))-($2*sin($3))); } { print xn } ')
dry=$(echo $xs $ys $ang |awk '{ yn=(($1*sin($3))+($2*cos($3))); } { print yn } ')

#####create output file with perendicular displacement
awk -v xx=$drx -v yy=$dry '{if($1 == "ATOM" || $1 == "HETATM")  printf("%s%8.3f%8.3f%8.3f%s\n",substr($0,1,30), substr($0,31,8)+xx,substr($0,39,8)+yy,substr($0,47,8),substr($0,55,100)); else print $0 }' $2 > perp_${3}_${file2}

