#Separate a protein structure away from a reference structure

#Lin Li

#This tool displaces a protein structure away from a reference one in a straight line. 
#Usage: ./separate2pdb_pqr.sh {file1} {file2} {displacement}
#file1 is the reference structure. file2 is the structure to be dispalced. The structures must be in pdb or pqr format.
#displacement is the desired distance for file 2 to be shifted. 
#The output will have the form of: 
#
#             "  sep_{displacement}_{file2}  " 
#
#The script " masscentpdb_pqr.sh " MUST be in the same folder as this tool.
#
 
if [ $# != 3 ]
then
        echo "can't run, it need parameter... "
        echo "Usage: ./seperate2pdbs.sh file1 file2 dist"
        exit
fi
file1=$1
file2=$2
dist=$3

#####finding the center of mass of both structures

x1=$(./masscentpdb_pqr.sh $1 |awk '{print $1}')
y1=$(./masscentpdb_pqr.sh $1 |awk '{print $2}')
z1=$(./masscentpdb_pqr.sh $1 |awk '{print $3}')

x2=$(./masscentpdb_pqr.sh $2 |awk '{print $1}')
y2=$(./masscentpdb_pqr.sh $2 |awk '{print $2}')
z2=$(./masscentpdb_pqr.sh $2 |awk '{print $3}')

#####calculating the normalization factor
n=$(echo $x1 $y1 $z1 $x2 $y2 $z2 | awk '{print (($1-$4)^2+($2-$5)^2+($3-$6)^2)^0.5}')

dnx=$(echo $x1 $x2 $n|awk '{print ($2-$1)/$3 }' )
dny=$(echo $y1 $y2 $n|awk '{print ($2-$1)/$3 }' )
dnz=$(echo $z1 $z2 $n|awk '{print ($2-$1)/$3 }' )

#####finding final displacement

dx=$(echo $dist $dnx |awk '{print $1*$2}')
dy=$(echo $dist $dny |awk '{print $1*$2}')
dz=$(echo $dist $dnz |awk '{print $1*$2}')

##### applying final displacement to the intial position of file2

awk -v xx=$dx -v yy=$dy -v zz=$dz '{if($1 == "ATOM" || $1 == "HETATM")  printf("%s%8.3f%8.3f%8.3f%s\n",substr($0,1,30), substr($0,31,8)+xx,substr($0,39,8)+yy,substr($0,47,8)+zz,substr($0,55,100)); else print $0 }' $2 > sep_${dist}_${file2}
